=== added directory 'plugin/regex_policy'
=== added file 'plugin/regex_policy/module.cc'
--- plugin/regex_policy/module.cc	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/module.cc	2011-03-03 01:55:10 +0000
@@ -0,0 +1,289 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Monty Taylor <mordred@inaugust.com>
+ *  Copyright (C) 2011 Canonical, Ltd.
+ *  Author: Clint Byrum <clint.byrum@canonical.com>
+ *
+ *  Copied from simple_user_policy
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include <config.h>
+
+#include <drizzled/plugin/authorization.h>
+#include <drizzled/module/option_map.h>
+
+#include "policy.h"
+
+namespace po= boost::program_options;
+
+using namespace std;
+using namespace drizzled;
+
+namespace regex_policy
+{
+
+static int init(module::Context &context)
+{
+  const module::option_map &vm= context.getOptions();
+
+  Policy *policy= new (nothrow) Policy(fs::path(vm["policy"].as<string>()));
+  if (policy == NULL or not policy->loadFile())
+  {
+    errmsg_printf(error::ERROR, _("Could not load regex policy file: %s\n"),
+                  (policy ? policy->getError().str().c_str() : _("Unknown")));
+    if (policy)
+    {
+      delete policy;
+    }
+    return 1;
+  }
+
+  context.add(policy);
+  context.registerVariable(new sys_var_const_string_val("policy", vm["policy"].as<string>()));
+
+  return 0;
+}
+
+static void init_options(drizzled::module::option_context &context)
+{
+  context("policy",
+      po::value<string>()->default_value(DEFAULT_POLICY_FILE.string()),
+      N_("File to load for regex authorization policies"));
+}
+
+bool Policy::loadFile()
+{
+  ifstream file(policy_file.string().c_str());
+  boost::regex *comment_re= NULL;
+  boost::regex *empty_re= NULL;
+  boost::regex *table_matches_re= NULL;
+  boost::regex *process_matches_re= NULL;
+  boost::regex *schema_matches_re= NULL;
+
+  try
+  {
+    comment_re= new boost::regex(comment_regex);
+    empty_re= new boost::regex(empty_regex);
+    table_matches_re= new boost::regex(table_match_regex);
+    process_matches_re= new boost::regex(process_match_regex);
+    schema_matches_re= new boost::regex(schema_match_regex);
+  } 
+  catch (const std::exception &e)
+  {
+    if (comment_re)
+    {
+      delete comment_re;
+    }
+    if (empty_re)
+    {
+      delete empty_re;
+    }
+    if (table_matches_re)
+    {
+      delete table_matches_re;
+    }
+    if (process_matches_re)
+    {
+      delete process_matches_re;
+    }
+    if (schema_matches_re)
+    {
+      delete schema_matches_re;
+    }
+    error << e.what();
+    return false;
+  }
+
+  if (! file.is_open())
+  {
+    error << "Unable to open regex policy file: " << policy_file.string();
+    return false;
+  }
+
+  int lines= 0;
+  try
+  {
+    while (! file.eof())
+    {
+      ++lines;
+      string line;
+      getline(file, line);
+      if (boost::regex_match(line, *comment_re))
+      {
+        continue;
+      }
+      if (boost::regex_match(line, *empty_re))
+      {
+        continue;
+      }
+      boost::smatch matches;
+      PolicyItemList *policies;
+      if (boost::regex_match(line, matches, *table_matches_re, boost::match_extra))
+      {
+        policies= &table_policies;
+      }
+      else if (boost::regex_match(line, matches, *process_matches_re, boost::match_extra))
+      {
+        policies= &process_policies;
+      }
+      else if (boost::regex_match(line, matches, *schema_matches_re, boost::match_extra))
+      {
+        policies= &schema_policies;
+      }
+      else
+      {
+        throw std::exception();
+      }
+      string user_regex;
+      string object_regex;
+      string action;
+      user_regex= matches[MATCH_REGEX_USER_POS];
+      object_regex= matches[MATCH_REGEX_OBJECT_POS];
+      action= matches[MATCH_REGEX_ACTION_POS];
+      PolicyItem *i;
+      try
+      {
+        i= new PolicyItem(user_regex, object_regex, action);
+      }
+      catch (const std::exception &e)
+      {
+        error << "Bad policy item: user=" << user_regex << " object=" << object_regex << " action=" << action;
+        throw std::exception();
+      }
+      policies->push_back(i);
+    }
+    return true;
+  }
+  catch (const std::exception &e)
+  {
+    /* On any non-EOF break, unparseable line */
+    error << "Unable to parse line " << lines << " of policy file " << policy_file.string() << ":" << e.what();
+    return false;
+  }
+}
+
+bool Policy::restrictObject(const drizzled::identifier::User &user_ctx,
+                                   const string &obj, const PolicyItemList &policies,
+                                   CheckMap **check_cache)
+{
+  CheckItem c(user_ctx.username(), obj, check_cache);
+  if (!c.hasCachedResult())
+  {
+    PolicyItemList::const_iterator m= find_if(policies.begin(), policies.end(), c);
+    if (m != policies.end())
+    {
+      c.setCachedResult((*m)->isRestricted());
+    }
+    else
+    {
+      /* TODO: make default action configurable */
+      c.setCachedResult(false);
+    }
+  }
+  return c.getCachedResult();
+}
+
+bool Policy::restrictSchema(const drizzled::identifier::User &user_ctx,
+                                   drizzled::identifier::Schema::const_reference schema)
+{
+  return restrictObject(user_ctx, schema.getSchemaName(), schema_policies, &schema_check_cache);
+}
+
+bool Policy::restrictProcess(const drizzled::identifier::User &user_ctx,
+                                    const drizzled::identifier::User &session_ctx)
+{
+  return restrictObject(user_ctx, session_ctx.username(), process_policies, &process_check_cache);
+}
+
+bool Policy::restrictTable(drizzled::identifier::User::const_reference user_ctx,
+                             drizzled::identifier::Table::const_reference table)
+{
+  return restrictObject(user_ctx, table.getTableName(), table_policies, &table_check_cache);
+}
+
+bool CheckItem::operator()(PolicyItem *p)
+{
+  if (p->userMatches(user))
+  {
+    errmsg_printf(error::INSPECT, _("User %s matches regex\n"), user.c_str());
+    if (p->objectMatches(object))
+    {
+      errmsg_printf(error::INSPECT, _("Object %s matches regex %s (%s)\n"), 
+          object.c_str(),
+          p->getObject().c_str(),
+          p->getAction());
+      return true;
+    }
+    errmsg_printf(error::INSPECT, _("Object %s NOT restricted by regex %s (%s)\n"), 
+        object.c_str(),
+        p->getObject().c_str(),
+        p->getAction());
+  }
+  return false;
+}
+
+CheckItem::CheckItem(const std::string &user_in, const std::string &obj_in, CheckMap **check_cache_in)
+  : user(user_in), object(obj_in), has_cached_result(false), check_cache(check_cache_in)
+{
+  CheckMap::iterator check_val;
+  std::stringstream keystream;
+  keystream << user << "_" << object;
+  key= keystream.str();
+
+  /* using RCU to only need to lock when updating the cache */
+  if ((*check_cache) && (check_val= (*check_cache)->find(key)) != (*check_cache)->end())
+  {
+    setCachedResult(check_val->second);
+  }
+}
+
+void CheckItem::setCachedResult(bool result)
+{
+  // TODO: make the mutex per-cache
+  CheckMap *old_cache;
+  CheckMap *new_cache;
+  boost::mutex::scoped_lock lock(check_cache_mutex, boost::defer_lock);
+  lock.lock();
+
+  // Copy the current one
+  if (*check_cache)
+  {
+    new_cache= new CheckMap(**check_cache);
+  }
+  else
+  {
+    new_cache= new CheckMap();
+  }
+  // Update it
+  (*new_cache)[key]= result;
+  // Replace old
+  old_cache= *check_cache;
+  *check_cache= new_cache;
+
+  lock.unlock();
+  has_cached_result= true;
+  cached_result= result;
+
+  if (old_cache)
+  {
+    delete old_cache;
+  }
+}
+
+} /* namespace regex_policy */
+
+DRIZZLE_PLUGIN(regex_policy::init, NULL, regex_policy::init_options);

=== added file 'plugin/regex_policy/plugin.ini'
--- plugin/regex_policy/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/plugin.ini	2011-03-03 01:55:10 +0000
@@ -0,0 +1,8 @@
+[plugin]
+title=Regex Policy
+description=Authorization plugin which implements a regex to object mapping
+version=1.0
+author=Clint Byrum <clint.byrum@canonical.com>
+license=PLUGIN_LICENSE_GPL
+sources=module.cc
+headers=policy.h

=== added file 'plugin/regex_policy/policy.h'
--- plugin/regex_policy/policy.h	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/policy.h	2011-03-03 01:55:10 +0000
@@ -0,0 +1,183 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2010 Monty Taylor <mordred@inaugust.com>
+ *  Copyright (C) 2011 Canonical, Ltd.
+ *  Author: Clint Byrum <clint.byrum@canonical.com>
+ *
+ *  Copied from simple_user_policy
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+
+#ifndef PLUGIN_REGEX_POLICY_POLICY_H
+#define PLUGIN_REGEX_POLICY_POLICY_H
+
+#include <iostream>
+#include <fstream>
+
+#include <boost/regex.hpp>
+#include <boost/unordered_map.hpp>
+#include <boost/thread/mutex.hpp>
+
+#include <drizzled/configmake.h>
+#include <drizzled/plugin/authorization.h>
+
+namespace fs= boost::filesystem;
+
+namespace regex_policy
+{
+
+static const fs::path DEFAULT_POLICY_FILE= SYSCONFDIR "/drizzle.policy";
+
+static const char *comment_regex = "^[[:space:]]*#.*$";
+static const char *empty_regex = "^[[:space:]]*$";
+static const char *table_match_regex = "^([^ ]+) table\\=([^ ]+) (ACCEPT|DENY)$";
+static const char *process_match_regex = "^([^ ]+) process\\=([^ ]+) (ACCEPT|DENY)$";
+static const char *schema_match_regex = "^([^ ]+) schema\\=([^ ]+) (ACCEPT|DENY)$";
+/* These correspond to the parenthesis above and must stay in sync */
+static const int MATCH_REGEX_USER_POS= 1;
+static const int MATCH_REGEX_OBJECT_POS= 2;
+static const int MATCH_REGEX_ACTION_POS= 3;
+
+typedef enum 
+{
+  POLICY_ACCEPT,
+  POLICY_DENY
+} PolicyAction;
+
+class PolicyItem
+{
+  const std::string user;
+  const std::string object;
+  const boost::regex user_re;
+  const boost::regex object_re;
+  PolicyAction action;
+public:
+  PolicyItem(const std::string &u, const std::string &obj, const std::string &act) :
+    user(u),
+    object(obj),
+    user_re(u),
+    object_re(obj)
+  { 
+    if (act == "ACCEPT")
+    {
+      action = POLICY_ACCEPT;
+    }
+    else if (act == "DENY")
+    {
+      action = POLICY_DENY;
+    }
+    else
+    {
+      throw std::exception();
+    }
+  }
+  bool userMatches(std::string &str);
+  bool objectMatches(std::string &object_id);
+  bool isRestricted();
+  const std::string&getUser() const
+  {
+    return user;
+  }
+  const std::string&getObject() const
+  {
+    return object;
+  }
+  const char *getAction() const
+  {
+    return action == POLICY_ACCEPT ? "ACCEPT" : "DENY";
+  }
+};
+
+typedef std::list<PolicyItem *> PolicyItemList;
+typedef boost::unordered_map<std::string, bool> CheckMap;
+
+static boost::mutex check_cache_mutex;
+
+class CheckItem
+{
+  std::string user;
+  std::string object;
+  std::string key;
+  bool has_cached_result;
+  bool cached_result;
+  CheckMap **check_cache;
+public:
+  CheckItem(const std::string &u, const std::string &obj, CheckMap **check_cache);
+  bool operator()(PolicyItem *p);
+  bool hasCachedResult() const
+  {
+    return has_cached_result;
+  }
+  bool getCachedResult() const
+  {
+    return cached_result;
+  }
+  void setCachedResult(bool result);
+};
+
+inline bool PolicyItem::userMatches(std::string &str)
+{
+  return boost::regex_match(str, user_re);
+}
+
+inline bool PolicyItem::objectMatches(std::string &object_id)
+{
+  return boost::regex_match(object_id, object_re);
+}
+
+inline bool PolicyItem::isRestricted()
+{
+  return action == POLICY_DENY ? true : false;
+}
+
+class Policy :
+  public drizzled::plugin::Authorization
+{
+public:
+  Policy(const fs::path &f_path) :
+    drizzled::plugin::Authorization("Regex Policy"), policy_file(f_path), error(),
+    table_check_cache(NULL), schema_check_cache(NULL), process_check_cache(NULL)
+  { }
+
+  virtual bool restrictSchema(const drizzled::identifier::User &user_ctx,
+                              drizzled::identifier::Schema::const_reference schema);
+
+  virtual bool restrictProcess(const drizzled::identifier::User &user_ctx,
+                               const drizzled::identifier::User &session_ctx);
+
+  virtual bool restrictTable(drizzled::identifier::User::const_reference user_ctx,
+                             drizzled::identifier::Table::const_reference table);
+
+  bool loadFile();
+  std::stringstream &getError() { return error; }
+private:
+  bool restrictObject(const drizzled::identifier::User &user_ctx,
+                                   const std::string &obj, const PolicyItemList &policies,
+                                   CheckMap **check_cache);
+  fs::path policy_file;
+  std::stringstream error;
+  PolicyItemList table_policies;
+  PolicyItemList schema_policies;
+  PolicyItemList process_policies;
+  CheckMap *table_check_cache;
+  CheckMap *schema_check_cache;
+  CheckMap *process_check_cache;
+};
+
+} /* namespace regex_policy */
+
+#endif /* PLUGIN_REGEX_POLICY_POLICY_H */

=== added directory 'plugin/regex_policy/tests'
=== added directory 'plugin/regex_policy/tests/r'
=== added file 'plugin/regex_policy/tests/r/basic.result'
--- plugin/regex_policy/tests/r/basic.result	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/tests/r/basic.result	2011-03-03 01:55:10 +0000
@@ -0,0 +1,72 @@
+create schema user1;
+create schema user2;
+SELECT SCHEMA_NAME FROM DATA_DICTIONARY.SCHEMAS ORDER BY SCHEMA_NAME;
+SCHEMA_NAME
+DATA_DICTIONARY
+INFORMATION_SCHEMA
+mysql
+test
+user1
+user2
+use user2;
+create table t1 (kill_id int);
+insert into t1 values(connection_id());
+SELECT * from user1.dont_exist;
+ERROR 42S02: Unknown table 'user1.dont_exist'
+SELECT SCHEMA_NAME FROM DATA_DICTIONARY.SCHEMAS ORDER BY SCHEMA_NAME;
+SCHEMA_NAME
+DATA_DICTIONARY
+INFORMATION_SCHEMA
+user1
+SELECT * from user2.dont_exist;
+ERROR 42000: Access denied for user 'user1' to schema 'user2'
+create schema authorize_fail;
+ERROR 42000: Access denied for user 'user1' to schema 'authorize_fail'
+drop schema user2;
+ERROR 42000: Access denied for user 'user1' to schema 'user2'
+create table t1 (kill_id int);
+insert into t1 values(connection_id());
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+USERNAME	DB
+user1	user1
+user1	user1
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+IF(((@id := kill_id) - kill_id), "NO", "YES")
+YES
+kill @id;
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+IF(((@id := kill_id) - kill_id), "NO", "YES")
+YES
+select @id != connection_id();
+@id != connection_id()
+1
+update t1 set kill_id = connection_id();
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+USERNAME	DB
+user2	user2
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+IF(((@id := kill_id) - kill_id), "NO", "YES")
+YES
+kill @id;
+ERROR HY000: Unknown session id: #
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+USERNAME	DB
+root	user2
+user1	user1
+user1	user1
+user2	user2
+use user1;
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+IF(((@id := kill_id) - kill_id), "NO", "YES")
+YES
+kill @id;
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+IF(((@id := kill_id) - kill_id), "NO", "YES")
+YES
+select @id != connection_id();
+@id != connection_id()
+1
+connect(localhost,authz,,authz_no,MASTER_PORT,);
+ERROR 42000: Access denied for user 'authz' to schema 'authz_no'
+drop schema user1;
+drop schema user2;

=== added directory 'plugin/regex_policy/tests/t'
=== added file 'plugin/regex_policy/tests/t/basic-master.opt'
--- plugin/regex_policy/tests/t/basic-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/tests/t/basic-master.opt	2011-03-03 01:55:10 +0000
@@ -0,0 +1,1 @@
+--plugin-add=regex_policy --regex-policy.policy=$TOP_SRCDIR/plugin/regex_policy/tests/t/basic.policy --verbose=INSPECT

=== added file 'plugin/regex_policy/tests/t/basic.policy'
--- plugin/regex_policy/tests/t/basic.policy	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/tests/t/basic.policy	2011-03-03 01:55:10 +0000
@@ -0,0 +1,15 @@
+# This is a comment line and should be skipped
+.+ schema=DATA_DICTIONARY ACCEPT
+.+ schema=INFORMATION_SCHEMA ACCEPT
+.+ schema=data_dictionary ACCEPT
+.+ schema=information_schema ACCEPT
+root table=.+ ACCEPT
+root schema=.+ ACCEPT
+root process=.+ ACCEPT
+user1 schema=user1 ACCEPT
+user2 schema=user2 ACCEPT
+user1 process=user1 ACCEPT
+user2 process=user2 ACCEPT
+# Default to denying everything
+.+ schema=.+ DENY
+.+ process=.+ DENY

=== added file 'plugin/regex_policy/tests/t/basic.test'
--- plugin/regex_policy/tests/t/basic.test	1970-01-01 00:00:00 +0000
+++ plugin/regex_policy/tests/t/basic.test	2011-03-03 01:55:10 +0000
@@ -0,0 +1,130 @@
+# Check for error if no parameter provided
+create schema user1;
+create schema user2;
+SELECT SCHEMA_NAME FROM DATA_DICTIONARY.SCHEMAS ORDER BY SCHEMA_NAME;
+
+# Set up a table to be able to test not being able to kill other people
+use user2;
+create table t1 (kill_id int);
+insert into t1 values(connection_id());
+
+# Test that we get a normal don't exist error for things that don't exist
+--error ER_TABLE_UNKNOWN
+SELECT * from user1.dont_exist;
+
+# Connect as user1 - should only see information_schema, user1 and
+# data_dictionary
+# Also tests that we are able to read data_dictionary, without which fail
+# would happen
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+connect (should_succeed,localhost,user1,,user1,,);
+connection should_succeed;
+SELECT SCHEMA_NAME FROM DATA_DICTIONARY.SCHEMAS ORDER BY SCHEMA_NAME;
+
+# Test that we get blocked on not being allowed to see user2 at all before
+# we get blocked on the table not existing
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+--replace_regex /@'.*?'/@'LOCALHOST'/
+--error ER_DBACCESS_DENIED_ERROR
+SELECT * from user2.dont_exist;
+
+# Test that we can't create a schema that isn't named the same as we are
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+--replace_regex /@'.*?'/@'LOCALHOST'/
+--error ER_DBACCESS_DENIED_ERROR
+create schema authorize_fail;
+
+# Test that we can't drop a schema that isn't named the same as we are
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+--replace_regex /@'.*?'/@'LOCALHOST'/
+--error ER_DBACCESS_DENIED_ERROR
+drop schema user2;
+
+# Set up a table to test that we can kill other versions of us
+create table t1 (kill_id int);
+insert into t1 values(connection_id());
+
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+connect (con1,localhost,user1,,user1,);
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+connect (con2,localhost,user2,,user2,);
+connection con1;
+
+# Check that we don't see other people's connections
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+
+# Check that we can kill a process that is owned by our user
+--disable_reconnect
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+kill @id;
+
+connection should_succeed;
+--sleep 2
+
+--disable_query_log
+--disable_result_log
+# One of the following statements should fail
+--error EE_OK,EE_BADCLOSE,EE_UNKNOWN_CHARSET,EE_CANT_SYMLINK
+select 1;
+--error EE_OK,EE_BADCLOSE,EE_UNKNOWN_CHARSET,EE_CANT_SYMLINK
+select 1;
+--enable_query_log
+--enable_result_log
+--enable_reconnect
+
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+select @id != connection_id();
+
+# Set the table to our current id now
+update t1 set kill_id = connection_id();
+
+# Test that we cannot kill a process owned by someone else
+connection con2;
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+
+--replace_regex /Unknown session id: [0-9]+/Unknown session id: #/
+--error ER_NO_SUCH_THREAD
+kill @id;
+
+# Test that root can see everybody
+connection default;
+SELECT USERNAME, DB FROM DATA_DICTIONARY.PROCESSLIST ORDER BY USERNAME, DB;
+
+# Test that root can kill someone else
+use user1;
+--disable_reconnect
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+kill @id;
+
+connection should_succeed;
+--sleep 2
+
+--disable_query_log
+--disable_result_log
+# One of the following statements should fail
+--error EE_OK,EE_BADCLOSE,EE_UNKNOWN_CHARSET,EE_CANT_SYMLINK
+select 1;
+--error EE_OK,EE_BADCLOSE,EE_UNKNOWN_CHARSET,EE_CANT_SYMLINK
+select 1;
+--enable_query_log
+--enable_result_log
+--enable_reconnect
+
+select IF(((@id := kill_id) - kill_id), "NO", "YES") from t1;
+select @id != connection_id();
+
+
+# Test failing initial connection
+--replace_result $MASTER_MYSOCK MASTER_SOCKET $MASTER_MYPORT MASTER_PORT
+--replace_regex /@'.*?'/@'LOCALHOST'/
+--error ER_DBACCESS_DENIED_ERROR
+connect (should_fail,localhost,authz,,authz_no,,);
+
+# Clean up after ourselves
+connection default;
+drop schema user1;
+drop schema user2;
+disconnect con1;
+disconnect con2;
+disconnect should_succeed;

